@@ -10,6 +10,7 @@ from .sslManager import *
 from .tlsAdapter import *
 from .urlManager import *
 from .userAgentManager import *
-from .seleniumManager import *
+from .seleneumManager import *
 from .videoDownloader import *
 from .middleManager import *
+seleniumManager = seleneumManager
