@@ -2,11 +2,7 @@ from ...abstract_webtools import *
 from ..urlManager import *
 from ..requestManager import *
 from ..soupManager import *
-def get_url(url=None,url_mgr=None):
-    url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
-    return url_mgr.url
-def get_url_mgr(url=None,url_mgr=None):
-    return url_mgr or urlManager(url)
+
 class linkManager:
     """
     LinkManager is a class for managing and extracting links and image links from a web page.
@@ -43,12 +39,34 @@ class linkManager:
         - The LinkManager class helps manage and extract links and image links from web pages.
         - The class provides flexibility in specifying criteria for link extraction.
     """
-    def __init__(self,url=None,source_code=None,url_mgr=None,req_mgr=None,soup_manager=None,image_link_tags='img',img_link_attrs='src',link_tags='a',link_attrs='href',strict_order_tags=False,img_attr_value_desired=None,img_attr_value_undesired=None,link_attr_value_desired=None,link_attr_value_undesired=None,associated_data_attr=["data-title",'alt','title'],get_img=["data-title",'alt','title']):
+    def __init__(self,
+                 url=None,
+                 source_code=None,
+                 soup=None,
+                 url_mgr=None,
+                 req_mgr=None,
+                 soup_mgr=None,
+                 parse_type=None,
+                 image_link_tags='img',
+                 img_link_attrs='src',
+                 link_tags='a',
+                 link_attrs='href',
+                 strict_order_tags=False,
+                 img_attr_value_desired=None,
+                 img_attr_value_undesired=None,
+                 link_attr_value_desired=None,
+                 link_attr_value_undesired=None,
+                 associated_data_attr=["data-title",'alt','title'],
+                 get_img=["data-title",'alt','title']
+                 ):
+
         self.url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
-        self.url=self.url_mgr.url
-        self.req_mgr = get_req_mgr(req_mgr=req_mgr,url=self.url,url_mgr=self.url_mgr,source_code=source_code)
-        self.source_code = source_code or self.req_mgr.source_code or self.req_mgr.source_code_bytes
-        self.soup_mgr = get_soup_mgr(req_mgr=self.req_mgr,url_mgr=self.url_mgr,source_code = source_code)
+        self.url = get_url(url=url,url_mgr=self.url_mgr)
+        self.req_mgr = get_req_mgr(url=self.url,url_mgr=self.url_mgr,source_code=source_code,req_mgr=req_mgr)
+        self.source_code = get_source(url=self.url,url_mgr=self.url_mgr,source_code=source_code,req_mgr=self.req_mgr)
+        self.soup_mgr = get_soup_mgr(url=self.url,url_mgr=self.url_mgr,source_code=self.source_code,req_mgr=self.req_mgr,soup_mgr=soup_mgr,soup=soup,parse_type=parse_type)
+        
+        self.soup = get_soup(url=self.url,url_mgr=self.url_mgr,req_mgr=self.req_mgr,source_code=self.source_code,soup_mgr=self.soup_mgr)
 
         self.strict_order_tags=strict_order_tags
         self.image_link_tags=image_link_tags
@@ -71,6 +89,7 @@ class linkManager:
                                                            attr_value_undesired=self.link_attr_value_undesired,
                                                            associated_data_attr=self.associated_data_attr,
                                                            get_img=get_img)
+
     def re_initialize(self):
         self.all_desired_image_links=self.find_all_desired_links(tag=self.image_link_tags,attr=self.img_link_attrs,strict_order_tags=self.strict_order_tags,attr_value_desired=self.img_attr_value_desired,attr_value_undesired=self.img_attr_value_undesired)
         self.all_desired_links=self.find_all_desired_links(tag=self.link_tags,attr=self.link_attrs,strict_order_tags=self.strict_order_tags,attr_value_desired=self.link_attr_value_desired,attr_value_undesired=self.link_attr_value_undesired,associated_data_attr=self.associated_data_attr,get_img=self.get_img)
@@ -164,19 +183,25 @@ class linkManager:
                 if url not in domains_ls and comp_domain == domain:
                     domains_ls.append(url)
         return domains_ls
+
     def find_all_desired_links(self,tag='img', attr='src',attr_value_desired=None,strict_order_tags=False,attr_value_undesired=None,associated_data_attr=None,all_desired=None,get_img=None):
         all_desired = all_desired or self.find_all_desired(tag=tag,attr=attr,strict_order_tags=strict_order_tags,attr_value_desired=attr_value_desired,attr_value_undesired=attr_value_undesired,associated_data_attr=associated_data_attr,get_img=get_img)
         assiciated_attrs = all_desired[-1]
         valid_assiciated_attrs = []
         desired_links=[]
         for i,attr in enumerate(all_desired[:-1]):
-            valid_attr=self.url_mgr.make_valid(attr,self.url_mgr.protocol+'://'+self.url_mgr.domain) 
-            if valid_attr:
-                desired_links.append(valid_attr)
-                valid_assiciated_attrs.append(assiciated_attrs[i])
-                valid_assiciated_attrs[-1]["link"]=valid_attr
+
+            self.url_mgr.domain = self.url_mgr.domain or ''
+
+            self.url_mgr.protocol = self.url_mgr.protocol or 'https'
+ 
+            if attr:
+                valid_attr=self.url_mgr.make_valid(attr,self.url_mgr.protocol+'://'+self.url_mgr.domain) 
+                if valid_attr:
+                    desired_links.append(valid_attr)
+                    valid_assiciated_attrs.append(assiciated_attrs[i])
+                    valid_assiciated_attrs[-1]["link"]=valid_attr
         desired_links.append(valid_assiciated_attrs)
         return desired_links
 
-
     
